import drawsvg as draw
from math import cos, sin, sqrt, pi

class Figure:
    def __init__(self, largeur, hauteur, origine=(0, 0), animation=None):
        self.largeur = largeur
        self.hauteur = hauteur
        self.origine = (origine[0], -origine[1]-hauteur)
        self.animation = None
        self.fig = draw.Drawing(self.largeur, self.hauteur, origin=self.origine)
    
    def efface(self):
        self.fig = draw.Drawing(self.largeur, self.hauteur, origin=self.origine)
        
    def ajoute(self, objet):
        if self.animation is not None:
            duree, zooms, repetition = self.animation
            objet.append_anim(draw.AnimateTransform('scale', duree, zooms, repeatCount=repetition))
        self.fig.append(objet)
    
    def modifie_taille(self, largeur, hauteur, origine=None):
        if origine=='center':
            self.origine = (self.origine[0]-largeur/2, self.origine[1]+hauteur/2)
        elif origine is not None:
            self.origine = (origine[0], -origine[1]-hauteur)
        else:
            self.origine = (self.origine[0], self.origine[1]+self.hauteur-hauteur)
        self.largeur = largeur
        self.hauteur = hauteur
        elements = self.fig.all_elements()        
        self.fig = draw.Drawing(self.largeur, self.hauteur, origin=self.origine)
        for e in elements:
            self.fig.append(e)
    
    def modifie_centre(self, origine):
        self.origine = (origine[0], -origine[1]-self.hauteur)
        elements = self.fig.all_elements()
        self.fig = draw.Drawing(self.largeur, self.hauteur, origin=self.origine)
        for e in elements:
            self.fig.append(e)
    
    def enregistre(self, nom):
        self.fig.save_svg(nom)
    
    def zoom(self, duree, zooms, repetition):
        self.animation = (duree, zooms, repetition)
        
        
    
    
figure = Figure(400, 400)       

def rotation(angle, centre=None):
    # il faut donner un angle en degré dans le sens horaire
    angle = str(-180*angle/pi)
    if centre is None:
        return f"rotate({angle})"
    else:
        x, y = centre
        return f"rotate({angle},{x},{-y})"


def cercle(centre, rayon, remplissage='none', epaisseur=2, couleur='black'):
    """ Trace un cercle de centre et de rayon donnés. """
    x, y = centre
    c = draw.Circle(x, -y, rayon,
                fill=remplissage,
                stroke_width=epaisseur,
                stroke=couleur)
    figure.ajoute(c)

def translation(point, distance, angle):
    """ Renvoie l'image de 'point' par
        la translation de vecteur de norme 'distance'
        et qui forme un angle 'angle' avec l'axe des abscisses."""
    x, y = point
    return (x + distance*cos(-angle),  y - distance*sin(-angle))

def segment(point1, point2, couleur='black', epaisseur=1):
    """ Trace un segment entre les deux points """
    x1, y1 = point1
    x2, y2 = point2
    s = draw.Line(x1, -y1, x2, -y2,
                  stroke=couleur,
                  stroke_width=epaisseur)
    figure.ajoute(s)

def segments(liste_points, fermer=True, couleur='black', epaisseur=1, remplissage='none'):
    liste_coordonnees = []
    for x, y in liste_points:
        liste_coordonnees.extend([x, -y])
    s = draw.Lines(*liste_coordonnees, close=fermer, stroke=couleur, stroke_width=epaisseur, fill=remplissage)
    figure.ajoute(s)

def rectangle(coin_bas_gauche, longueur, hauteur, couleur='black', epaisseur=1, remplissage='none', transform=None):
    x, y = coin_bas_gauche
    r = draw.Rectangle(x, -y-hauteur, longueur, hauteur, stroke=couleur, stroke_width=epaisseur, fill=remplissage, transform=transform)
    figure.ajoute(r)